package handlers

import (
    "encoding/json"
    "log"
    "net/http"
    "assignment1/models"
    "assignment1/services"
    "fmt"
    "time"
)


// BookCountHandler handles requests to the /librarystats/v1/bookcount/ endpoint.
func BookCountHandler(w http.ResponseWriter, r *http.Request) {
    // Extract query parameters for language
    query := r.URL.Query()
    language := query.Get("language")

    // Fetch book data from the Gutendex API
    booksResponse, err := services.FetchBooksByLanguage(language)
    if err != nil {
        http.Error(w, "Failed to fetch books: "+err.Error(), http.StatusInternalServerError)
        return
    }

    // Process the books to calculate the book count and author count
    bookCount := len(booksResponse.Results)
    authorCount := services.CalculateUniqueAuthors(booksResponse.Results)

    // Create a response object
    response := map[string]interface{}{
        "language":   language,
        "bookCount":  bookCount,
        "authorCount": authorCount,
    }

    // Write the response back as JSON
    w.Header().Set("Content-Type", "application/json")
    if err := json.NewEncoder(w).Encode(response); err != nil {
        http.Error(w, "Failed to encode response: "+err.Error(), http.StatusInternalServerError)
    }
}

func ReadershipHandler(w http.ResponseWriter, r *http.Request) {
    language := r.URL.Path[len("/librarystats/v1/readership/"):]

    log.Printf("ReadershipHandler: Processing language: %s", language)

    // Optional: Parse query parameters for limit if present
    query := r.URL.Query()
    limitParam := query.Get("limit")
    var limit int
    if limitParam != "" {
        fmt.Sscanf(limitParam, "%d", &limit)
        log.Printf("ReadershipHandler: Limit set to %d", limit)
    }

    books, err := services.FetchBooksByLanguage(language)
    if err != nil {
        log.Printf("ReadershipHandler: Error fetching books: %v", err)
        http.Error(w, err.Error(), http.StatusInternalServerError)
        return
    }
    log.Printf("ReadershipHandler: Fetched %d books", len(books.Results))

    countries, err := services.FetchCountriesByLanguage(language)
    if err != nil {
        log.Printf("ReadershipHandler: Error fetching countries: %v", err)
        http.Error(w, err.Error(), http.StatusInternalServerError)
        return
    }
    log.Printf("ReadershipHandler: Fetched %d countries", len(countries))

    var readershipDetails []models.ReadershipDetail
    var totalReadership int64 = 0
    for _, country := range countries {
        population, err := services.FetchPopulationByCountryCode(country.ISO3166_1_Alpha_2)
        if err != nil {
            log.Printf("ReadershipHandler: Error fetching population for country %s: %v", country.ISO3166_1_Alpha_2, err)
            http.Error(w, err.Error(), http.StatusInternalServerError)
            return
        }
        totalReadership += population

        readershipDetail := models.ReadershipDetail{
            Country:    country.Official_Name,
            ISOCode:    country.ISO3166_1_Alpha_2,
            Books:      len(books.Results),
            Authors:    services.CalculateUniqueAuthors(books.Results),
            Readership: population,
        }
        readershipDetails = append(readershipDetails, readershipDetail)

        if limit > 0 && len(readershipDetails) >= limit {
            log.Printf("ReadershipHandler: Limit reached, stopping processing")
            break
        }
    }

    log.Printf("ReadershipHandler: Total readership calculated: %d", totalReadership)

    w.Header().Set("Content-Type", "application/json")
    if err := json.NewEncoder(w).Encode(readershipDetails); err != nil {
        log.Printf("ReadershipHandler: Error encoding response: %v", err)
        http.Error(w, "Failed to encode response: "+err.Error(), http.StatusInternalServerError)
    } else {
        log.Printf("ReadershipHandler: Response successfully sent")
    }
}

func StatusHandler(w http.ResponseWriter, r *http.Request) {
    // Initialize a variable to track the start time of the service if not already done
    // This should ideally be done at the application start, not here
    // Assuming services.ServiceStartTime is the time when your service started

    // Define the URLs for the services to check
    gutendexURL := "http://129.241.150.113:8000/books/"
    languageURL := "http://129.241.150.113:3000/language2countries/"
    countriesURL := "http://129.241.150.113:8080/v3.1/"

    // Use the CheckServiceAvailability function to get the status codes
    gutendexStatus := services.CheckServiceAvailability(gutendexURL)
    languageStatus := services.CheckServiceAvailability(languageURL)
    countriesStatus := services.CheckServiceAvailability(countriesURL)

    // Calculate uptime
    uptime := int64(time.Since(services.ServiceStartTime).Seconds())

    // Create a response struct
    statusResponse := struct {
        GutendexAPI  int   `json:"gutendexapi"`
        LanguageAPI  int   `json:"languageapi"`
        CountriesAPI int   `json:"countriesapi"`
        Version      string `json:"version"`
        Uptime       int64  `json:"uptime"`
    }{
        GutendexAPI:  gutendexStatus,
        LanguageAPI:  languageStatus,
        CountriesAPI: countriesStatus,
        Version:      "v1",
        Uptime:       uptime,
    }

    // Set the header and encode the response as JSON
    w.Header().Set("Content-Type", "application/json")
    if err := json.NewEncoder(w).Encode(statusResponse); err != nil {
        http.Error(w, err.Error(), http.StatusInternalServerError)
        return
    }
}
